import flags
import numpy
import pydrake.common
import pydrake.geometry
import pydrake.geometry.optimization
import pydrake.math
import pydrake.multibody.parsing
import pydrake.multibody.plant
import pydrake.multibody.rational
import pydrake.multibody.tree
import pydrake.solvers
import pydrake.symbolic
import pydrake.systems.framework
import pydrake.trajectories
import scipy.sparse
import typing
from . import experimental as experimental
from pydrake.common.cpp_template import RobotDiagramBuilder_ as RobotDiagramBuilder_, RobotDiagram_ as RobotDiagram_
from typing import Any, Callable, ClassVar, overload

class BodyShapeDescription:
    @overload
    def __init__(self, shape: pydrake.geometry.Shape, X_BS: pydrake.math.RigidTransform, model_instance_name: str, body_name: str) -> None: ...
    @overload
    def __init__(self, other: BodyShapeDescription) -> None: ...
    def body_name(self) -> str: ...
    def model_instance_name(self) -> str: ...
    def pose_in_body(self) -> pydrake.math.RigidTransform: ...
    def shape(self) -> pydrake.geometry.Shape: ...
    def __copy__(self) -> BodyShapeDescription: ...
    def __deepcopy__(self, arg0: dict) -> BodyShapeDescription: ...

class CollisionChecker:
    def __init__(self, *args, **kwargs) -> None: ...
    def AddCollisionShape(self, group_name: str, description: BodyShapeDescription) -> bool: ...
    def AddCollisionShapeToBody(self, group_name: str, bodyA: pydrake.multibody.tree.RigidBody, shape: pydrake.geometry.Shape, X_AG: pydrake.math.RigidTransform) -> bool: ...
    def AddCollisionShapeToFrame(self, group_name: str, frameA: pydrake.multibody.tree.Frame, shape: pydrake.geometry.Shape, X_AG: pydrake.math.RigidTransform) -> bool: ...
    @overload
    def AddCollisionShapes(self, group_name: str, descriptions: list[BodyShapeDescription]) -> int: ...
    @overload
    def AddCollisionShapes(self, geometry_groups: dict[str, list[BodyShapeDescription]]) -> dict[str, int]: ...
    def CalcContextRobotClearance(self, model_context: CollisionCheckerContext, q: numpy.ndarray[numpy.float64[m, 1]], influence_distance: float) -> RobotClearance: ...
    def CalcRobotClearance(self, q: numpy.ndarray[numpy.float64[m, 1]], influence_distance: float, context_number: int | None = ...) -> RobotClearance: ...
    def CheckConfigCollisionFree(self, q: numpy.ndarray[numpy.float64[m, 1]], context_number: int | None = ...) -> bool: ...
    def CheckConfigsCollisionFree(self, configs: list[numpy.ndarray[numpy.float64[m, 1]]], parallelize: pydrake.common.Parallelism = ...) -> list[int]: ...
    def CheckContextConfigCollisionFree(self, model_context: CollisionCheckerContext, q: numpy.ndarray[numpy.float64[m, 1]]) -> bool: ...
    def CheckContextEdgeCollisionFree(self, model_context: CollisionCheckerContext, q1: numpy.ndarray[numpy.float64[m, 1]], q2: numpy.ndarray[numpy.float64[m, 1]]) -> bool: ...
    def CheckEdgeCollisionFree(self, q1: numpy.ndarray[numpy.float64[m, 1]], q2: numpy.ndarray[numpy.float64[m, 1]], context_number: int | None = ...) -> bool: ...
    def CheckEdgeCollisionFreeParallel(self, q1: numpy.ndarray[numpy.float64[m, 1]], q2: numpy.ndarray[numpy.float64[m, 1]], parallelize: pydrake.common.Parallelism = ...) -> bool: ...
    def CheckEdgesCollisionFree(self, edges: list[tuple[numpy.ndarray[numpy.float64[m, 1]], numpy.ndarray[numpy.float64[m, 1]]]], parallelize: pydrake.common.Parallelism = ...) -> list[int]: ...
    def ClassifyBodyCollisions(self, q: numpy.ndarray[numpy.float64[m, 1]], context_number: int | None = ...) -> list[RobotCollisionType]: ...
    def ClassifyContextBodyCollisions(self, model_context: CollisionCheckerContext, q: numpy.ndarray[numpy.float64[m, 1]]) -> list[RobotCollisionType]: ...
    def Clone(self) -> CollisionChecker: ...
    def ComputeConfigurationDistance(self, q1: numpy.ndarray[numpy.float64[m, 1]], q2: numpy.ndarray[numpy.float64[m, 1]]) -> float: ...
    def GetAllAddedCollisionShapes(self) -> dict[str, list[BodyShapeDescription]]: ...
    def GetFilteredCollisionMatrix(self) -> numpy.ndarray[numpy.int32[m, n]]: ...
    def GetLargestPadding(self) -> float: ...
    def GetNominalFilteredCollisionMatrix(self) -> numpy.ndarray[numpy.int32[m, n]]: ...
    @overload
    def GetPaddingBetween(self, bodyA_index: pydrake.multibody.tree.BodyIndex, bodyB_index: pydrake.multibody.tree.BodyIndex) -> float: ...
    @overload
    def GetPaddingBetween(self, bodyA: pydrake.multibody.tree.RigidBody, bodyB: pydrake.multibody.tree.RigidBody) -> float: ...
    def GetPaddingMatrix(self) -> numpy.ndarray[numpy.float64[m, n]]: ...
    def GetZeroConfiguration(self) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    def InterpolateBetweenConfigurations(self, q1: numpy.ndarray[numpy.float64[m, 1]], q2: numpy.ndarray[numpy.float64[m, 1]], ratio: float) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    @overload
    def IsCollisionFilteredBetween(self, bodyA_index: pydrake.multibody.tree.BodyIndex, bodyB_index: pydrake.multibody.tree.BodyIndex) -> bool: ...
    @overload
    def IsCollisionFilteredBetween(self, bodyA: pydrake.multibody.tree.RigidBody, bodyB: pydrake.multibody.tree.RigidBody) -> bool: ...
    @overload
    def IsPartOfRobot(self, body: pydrake.multibody.tree.RigidBody) -> bool: ...
    @overload
    def IsPartOfRobot(self, body_index: pydrake.multibody.tree.BodyIndex) -> bool: ...
    def MakeStandaloneConfigurationDistanceFunction(self) -> Callable[[numpy.ndarray[numpy.float64[m, 1]], numpy.ndarray[numpy.float64[m, 1]]], float]: ...
    def MakeStandaloneConfigurationInterpolationFunction(self) -> Callable[[numpy.ndarray[numpy.float64[m, 1]], numpy.ndarray[numpy.float64[m, 1]], float], numpy.ndarray[numpy.float64[m, 1]]]: ...
    def MakeStandaloneModelContext(self) -> CollisionCheckerContext: ...
    def MaxContextNumDistances(self, model_context: CollisionCheckerContext) -> int: ...
    def MaxNumDistances(self, context_number: int | None = ...) -> int: ...
    def MaybeGetUniformRobotEnvironmentPadding(self) -> float | None: ...
    def MaybeGetUniformRobotRobotPadding(self) -> float | None: ...
    def MeasureContextEdgeCollisionFree(self, model_context: CollisionCheckerContext, q1: numpy.ndarray[numpy.float64[m, 1]], q2: numpy.ndarray[numpy.float64[m, 1]]) -> EdgeMeasure: ...
    def MeasureEdgeCollisionFree(self, q1: numpy.ndarray[numpy.float64[m, 1]], q2: numpy.ndarray[numpy.float64[m, 1]], context_number: int | None = ...) -> EdgeMeasure: ...
    def MeasureEdgeCollisionFreeParallel(self, q1: numpy.ndarray[numpy.float64[m, 1]], q2: numpy.ndarray[numpy.float64[m, 1]], parallelize: pydrake.common.Parallelism = ...) -> EdgeMeasure: ...
    def MeasureEdgesCollisionFree(self, edges: list[tuple[numpy.ndarray[numpy.float64[m, 1]], numpy.ndarray[numpy.float64[m, 1]]]], parallelize: pydrake.common.Parallelism = ...) -> list[EdgeMeasure]: ...
    def PerformOperationAgainstAllModelContexts(self, operation: Callable[[RobotDiagram, CollisionCheckerContext], None]) -> None: ...
    @overload
    def RemoveAllAddedCollisionShapes(self, group_name: str) -> None: ...
    @overload
    def RemoveAllAddedCollisionShapes(self) -> None: ...
    def SetCollisionFilterMatrix(self, filter_matrix: numpy.ndarray[numpy.int32[m, n]]) -> None: ...
    @overload
    def SetCollisionFilteredBetween(self, bodyA_index: pydrake.multibody.tree.BodyIndex, bodyB_index: pydrake.multibody.tree.BodyIndex, filter_collision: bool) -> None: ...
    @overload
    def SetCollisionFilteredBetween(self, bodyA: pydrake.multibody.tree.RigidBody, bodyB: pydrake.multibody.tree.RigidBody, filter_collision: bool) -> None: ...
    def SetCollisionFilteredWithAllBodies(self, body_index: pydrake.multibody.tree.BodyIndex) -> None: ...
    def SetConfigurationDistanceFunction(self, distance_function: Callable[[numpy.ndarray[numpy.float64[m, 1]], numpy.ndarray[numpy.float64[m, 1]]], float]) -> None: ...
    def SetConfigurationInterpolationFunction(self, interpolation_function: Callable[[numpy.ndarray[numpy.float64[m, 1]], numpy.ndarray[numpy.float64[m, 1]], float], numpy.ndarray[numpy.float64[m, 1]]]) -> None: ...
    def SetDistanceAndInterpolationProvider(self, provider: DistanceAndInterpolationProvider) -> None: ...
    def SetPaddingAllRobotEnvironmentPairs(self, padding: float) -> None: ...
    def SetPaddingAllRobotRobotPairs(self, padding: float) -> None: ...
    @overload
    def SetPaddingBetween(self, bodyA_index: pydrake.multibody.tree.BodyIndex, bodyB_index: pydrake.multibody.tree.BodyIndex, padding: float) -> None: ...
    @overload
    def SetPaddingBetween(self, bodyA: pydrake.multibody.tree.RigidBody, bodyB: pydrake.multibody.tree.RigidBody, padding: float) -> None: ...
    def SetPaddingMatrix(self, collision_padding: numpy.ndarray[numpy.float64[m, n]]) -> None: ...
    def SetPaddingOneRobotBodyAllEnvironmentPairs(self, body_index: pydrake.multibody.tree.BodyIndex, padding: float) -> None: ...
    def SupportsParallelChecking(self) -> bool: ...
    def UpdateContextPositions(self, model_context: CollisionCheckerContext, q: numpy.ndarray[numpy.float64[m, 1]]) -> pydrake.systems.framework.Context: ...
    def UpdatePositions(self, q: numpy.ndarray[numpy.float64[m, 1]], context_number: int | None = ...) -> pydrake.systems.framework.Context: ...
    def distance_and_interpolation_provider(self) -> DistanceAndInterpolationProvider: ...
    def edge_step_size(self) -> float: ...
    def get_body(self, body_index: pydrake.multibody.tree.BodyIndex) -> pydrake.multibody.tree.RigidBody: ...
    def model(self) -> RobotDiagram: ...
    def model_context(self, context_number: int | None = ...) -> CollisionCheckerContext: ...
    def num_allocated_contexts(self) -> int: ...
    def plant(self) -> pydrake.multibody.plant.MultibodyPlant: ...
    def plant_context(self, context_number: int | None = ...) -> pydrake.systems.framework.Context: ...
    def robot_model_instances(self) -> list[pydrake.multibody.tree.ModelInstanceIndex]: ...
    def set_edge_step_size(self, edge_step_size: float) -> None: ...
    def __copy__(self) -> CollisionChecker: ...
    def __deepcopy__(self, arg0: dict) -> CollisionChecker: ...

class CollisionCheckerContext:
    def __init__(self, model: RobotDiagram) -> None: ...
    def Clone(self) -> CollisionCheckerContext: ...
    def GetQueryObject(self) -> pydrake.geometry.QueryObject: ...
    def model_context(self) -> pydrake.systems.framework.Context: ...
    def plant_context(self) -> pydrake.systems.framework.Context: ...
    def scene_graph_context(self) -> pydrake.systems.framework.Context: ...
    def __copy__(self) -> CollisionCheckerContext: ...
    def __deepcopy__(self, arg0: dict) -> CollisionCheckerContext: ...

class CollisionCheckerParams:
    configuration_distance_function: Callable[[numpy.ndarray[numpy.float64[m, 1]], numpy.ndarray[numpy.float64[m, 1]]], float]
    distance_and_interpolation_provider: DistanceAndInterpolationProvider
    edge_step_size: float
    env_collision_padding: float
    implicit_context_parallelism: pydrake.common.Parallelism
    model: RobotDiagram
    robot_model_instances: list[pydrake.multibody.tree.ModelInstanceIndex]
    self_collision_padding: float
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, **kwargs) -> None: ...

class CommonSampledIrisOptions:
    configuration_space_margin: float
    containment_points: numpy.ndarray[numpy.float64[m, n]] | None
    delta: float
    epsilon: float
    max_iterations: int
    max_iterations_separating_planes: int
    max_separating_planes_per_iteration: int
    mixing_steps: int
    num_particles: int
    parallelism: pydrake.common.Parallelism
    prog_with_additional_constraints: pydrake.solvers.MathematicalProgram
    random_seed: int
    relative_termination_threshold: float
    relax_margin: bool
    remove_all_collisions_possible: bool
    require_sample_point_is_contained: bool
    sample_particles_in_parallel: bool
    tau: float
    termination_threshold: float
    verbose: bool
    def __init__(self) -> None: ...

class DirectCollocation(MultipleShooting):
    def __init__(self, system: pydrake.systems.framework.System, context: pydrake.systems.framework.Context, num_time_samples: int, minimum_time_step: float, maximum_time_step: float, input_port_index: pydrake.systems.framework.InputPortSelection | pydrake.systems.framework.InputPortIndex = ..., assume_non_continuous_states_are_fixed: bool = ..., prog: pydrake.solvers.MathematicalProgram = ...) -> None: ...

class DirectCollocationConstraint(pydrake.solvers.Constraint):
    def __init__(self, system: pydrake.systems.framework.System, context: pydrake.systems.framework.Context, input_port_index: pydrake.systems.framework.InputPortSelection | pydrake.systems.framework.InputPortIndex = ..., assume_non_continuous_states_are_fixed: bool = ...) -> None: ...

class DirectTranscription(MultipleShooting):
    class TimeStep:
        value: float
        def __init__(self, value: float) -> None: ...
    @overload
    def __init__(self, system: pydrake.systems.framework.System, context: pydrake.systems.framework.Context, num_time_samples: int, input_port_index: pydrake.systems.framework.InputPortSelection | pydrake.systems.framework.InputPortIndex = ...) -> None: ...
    @overload
    def __init__(self, system: pydrake.systems.framework.System, context: pydrake.systems.framework.Context, num_time_samples: int, fixed_time_step: DirectTranscription.TimeStep, input_port_index: pydrake.systems.framework.InputPortSelection | pydrake.systems.framework.InputPortIndex = ...) -> None: ...

class DistanceAndInterpolationProvider:
    def __init__(self, *args, **kwargs) -> None: ...
    def ComputeConfigurationDistance(self, arg0: numpy.ndarray[numpy.float64[m, 1]], arg1: numpy.ndarray[numpy.float64[m, 1]]) -> float: ...
    def InterpolateBetweenConfigurations(self, arg0: numpy.ndarray[numpy.float64[m, 1]], arg1: numpy.ndarray[numpy.float64[m, 1]], arg2: float) -> numpy.ndarray[numpy.float64[m, 1]]: ...

class DofMask:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, size: int, value: bool) -> None: ...
    @overload
    def __init__(self, values: list[bool]) -> None: ...
    def Complement(self) -> DofMask: ...
    @overload
    def GetFullToSelectedIndex(self) -> list[int | None]: ...
    @overload
    def GetFullToSelectedIndex(self) -> Any: ...
    @overload
    def GetFullToSelectedIndex(self) -> Any: ...
    def GetJoints(self, plant: pydrake.multibody.plant.MultibodyPlant) -> list[pydrake.multibody.tree.JointIndex]: ...
    @overload
    def GetSelectedToFullIndex(self) -> list[int]: ...
    @overload
    def GetSelectedToFullIndex(self) -> Any: ...
    def Intersect(self, other: DofMask) -> DofMask: ...
    @staticmethod
    def MakeFromModel(plant: pydrake.multibody.plant.MultibodyPlant, model_index: pydrake.multibody.tree.ModelInstanceIndex) -> DofMask: ...
    def Subtract(self, other: DofMask) -> DofMask: ...
    def Union(self, other: DofMask) -> DofMask: ...
    def count(self) -> int: ...
    def size(self) -> int: ...
    def __eq__(self, arg0: DofMask) -> bool: ...
    def __getitem__(self, arg0: int) -> bool: ...
    def __iter__(self) -> typing.Iterator[bool]: ...

class EdgeMeasure:
    @overload
    def __init__(self, distance: float, alpha: float) -> None: ...
    @overload
    def __init__(self, other: EdgeMeasure) -> None: ...
    def alpha(self) -> float: ...
    def alpha_or(self, default_value: float) -> float: ...
    def completely_free(self) -> bool: ...
    def distance(self) -> float: ...
    def partially_free(self) -> bool: ...
    def __copy__(self) -> EdgeMeasure: ...
    def __deepcopy__(self, arg0: dict) -> EdgeMeasure: ...

class GcsTrajectoryOptimization:
    class EdgesBetweenSubgraphs:
        def __init__(self, *args, **kwargs) -> None: ...
        def AddContinuityConstraints(self, continuity_order: int) -> None: ...
        @overload
        def AddEdgeConstraint(self, e: pydrake.symbolic.Formula, use_in_transcription: set[pydrake.geometry.optimization.GraphOfConvexSets.Transcription] = ...) -> None: ...
        @overload
        def AddEdgeConstraint(self, binding: pydrake.solvers.BindingTConstraintU, use_in_transcription: set[pydrake.geometry.optimization.GraphOfConvexSets.Transcription] = ...) -> None: ...
        @overload
        def AddEdgeCost(self, e: pydrake.symbolic.Expression, use_in_transcription: set[pydrake.geometry.optimization.GraphOfConvexSets.Transcription] = ...) -> None: ...
        @overload
        def AddEdgeCost(self, binding: pydrake.solvers.BindingTCostU, use_in_transcription: set[pydrake.geometry.optimization.GraphOfConvexSets.Transcription] = ...) -> None: ...
        def AddNonlinearDerivativeBounds(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]], derivative_order: int) -> None: ...
        def AddPathContinuityConstraints(self, continuity_order: int) -> None: ...
        def AddVelocityBounds(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]]) -> None: ...
        def AddZeroDerivativeConstraints(self, derivative_order: int) -> None: ...
        def Edges(self) -> list[pydrake.geometry.optimization.GraphOfConvexSets.Edge]: ...
        def edge_constituent_vertex_control_points(self) -> tuple[numpy.ndarray[object[m, n]], numpy.ndarray[object[m, n]]]: ...
        def edge_constituent_vertex_durations(self) -> tuple[pydrake.symbolic.Variable, pydrake.symbolic.Variable]: ...

    class Subgraph:
        def __init__(self, *args, **kwargs) -> None: ...
        def AddContinuityConstraints(self, continuity_order: int) -> None: ...
        @overload
        def AddEdgeConstraint(self, e: pydrake.symbolic.Formula, use_in_transcription: set[pydrake.geometry.optimization.GraphOfConvexSets.Transcription] = ...) -> None: ...
        @overload
        def AddEdgeConstraint(self, binding: pydrake.solvers.BindingTConstraintU, use_in_transcription: set[pydrake.geometry.optimization.GraphOfConvexSets.Transcription] = ...) -> None: ...
        @overload
        def AddEdgeCost(self, e: pydrake.symbolic.Expression, use_in_transcription: set[pydrake.geometry.optimization.GraphOfConvexSets.Transcription] = ...) -> None: ...
        @overload
        def AddEdgeCost(self, binding: pydrake.solvers.BindingTCostU, use_in_transcription: set[pydrake.geometry.optimization.GraphOfConvexSets.Transcription] = ...) -> None: ...
        def AddNonlinearDerivativeBounds(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]], derivative_order: int) -> None: ...
        def AddPathContinuityConstraints(self, continuity_order: int) -> None: ...
        @overload
        def AddPathEnergyCost(self, weight_matrix: numpy.ndarray[numpy.float64[m, n]]) -> None: ...
        @overload
        def AddPathEnergyCost(self, weight: float = ...) -> None: ...
        @overload
        def AddPathLengthCost(self, weight_matrix: numpy.ndarray[numpy.float64[m, n]]) -> None: ...
        @overload
        def AddPathLengthCost(self, weight: float = ...) -> None: ...
        def AddTimeCost(self, weight: float = ...) -> None: ...
        def AddVelocityBounds(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]]) -> None: ...
        @overload
        def AddVertexConstraint(self, e: pydrake.symbolic.Formula, use_in_transcription: set[pydrake.geometry.optimization.GraphOfConvexSets.Transcription] = ...) -> None: ...
        @overload
        def AddVertexConstraint(self, binding: pydrake.solvers.BindingTConstraintU, use_in_transcription: set[pydrake.geometry.optimization.GraphOfConvexSets.Transcription] = ...) -> None: ...
        @overload
        def AddVertexCost(self, e: pydrake.symbolic.Expression, use_in_transcription: set[pydrake.geometry.optimization.GraphOfConvexSets.Transcription] = ...) -> None: ...
        @overload
        def AddVertexCost(self, binding: pydrake.solvers.BindingTCostU, use_in_transcription: set[pydrake.geometry.optimization.GraphOfConvexSets.Transcription] = ...) -> None: ...
        def Edges(self) -> list[pydrake.geometry.optimization.GraphOfConvexSets.Edge]: ...
        def Vertices(self) -> list[pydrake.geometry.optimization.GraphOfConvexSets.Vertex]: ...
        def edge_constituent_vertex_control_points(self) -> tuple[numpy.ndarray[object[m, n]], numpy.ndarray[object[m, n]]]: ...
        def edge_constituent_vertex_durations(self) -> tuple[pydrake.symbolic.Variable, pydrake.symbolic.Variable]: ...
        def name(self) -> str: ...
        def order(self) -> int: ...
        def regions(self) -> object: ...
        def size(self) -> int: ...
        def vertex_control_points(self) -> numpy.ndarray[object[m, n]]: ...
        def vertex_duration(self) -> pydrake.symbolic.Variable: ...
    def __init__(self, num_positions: int, continuous_revolute_joints: list[int] = ...) -> None: ...
    def AddContinuityConstraints(self, continuity_order: int) -> None: ...
    def AddEdges(self, from_subgraph: GcsTrajectoryOptimization.Subgraph, to_subgraph: GcsTrajectoryOptimization.Subgraph, subspace: pydrake.geometry.optimization.ConvexSet = ..., edges_between_regions: list[tuple[int, int]] | None = ..., edge_offsets: list[numpy.ndarray[numpy.float64[m, 1]]] | None = ...) -> GcsTrajectoryOptimization.EdgesBetweenSubgraphs: ...
    def AddNonlinearDerivativeBounds(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]], derivative_order: int) -> None: ...
    def AddPathContinuityConstraints(self, continuity_order: int) -> None: ...
    @overload
    def AddPathEnergyCost(self, weight_matrix: numpy.ndarray[numpy.float64[m, n]]) -> None: ...
    @overload
    def AddPathEnergyCost(self, weight: float = ...) -> None: ...
    @overload
    def AddPathLengthCost(self, weight_matrix: numpy.ndarray[numpy.float64[m, n]]) -> None: ...
    @overload
    def AddPathLengthCost(self, weight: float = ...) -> None: ...
    def AddRegions(self, regions: list[pydrake.geometry.optimization.ConvexSet], edges_between_regions: list[tuple[int, int]], order: int, h_min: float = ..., h_max: float = ..., name: str = ..., edge_offsets: list[numpy.ndarray[numpy.float64[m, 1]]] | None = ...) -> GcsTrajectoryOptimization.Subgraph: ...
    def AddTimeCost(self, weight: float = ...) -> None: ...
    def AddVelocityBounds(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]]) -> None: ...
    def GetEdgesBetweenSubgraphs(self) -> list[GcsTrajectoryOptimization.EdgesBetweenSubgraphs]: ...
    @overload
    def GetGraphvizString(self, result: pydrake.solvers.MathematicalProgramResult = ..., options: pydrake.geometry.optimization.GcsGraphvizOptions = ...) -> str: ...
    @overload
    def GetGraphvizString(self, result: pydrake.solvers.MathematicalProgramResult = ..., show_slacks: bool = ..., show_vars: bool = ..., show_flows: bool = ..., show_costs: bool = ..., scientific: bool = ..., precision: int = ...) -> str: ...
    def GetSubgraphs(self) -> list[GcsTrajectoryOptimization.Subgraph]: ...
    @staticmethod
    def NormalizeSegmentTimes(trajectory: pydrake.trajectories.CompositeTrajectory) -> pydrake.trajectories.CompositeTrajectory: ...
    def RemoveSubgraph(self, subgraph: GcsTrajectoryOptimization.Subgraph) -> None: ...
    def SolveConvexRestriction(self, active_vertices: list[pydrake.geometry.optimization.GraphOfConvexSets.Vertex], options: pydrake.geometry.optimization.GraphOfConvexSetsOptions = ...) -> tuple[pydrake.trajectories.CompositeTrajectory, pydrake.solvers.MathematicalProgramResult]: ...
    def SolvePath(self, source: GcsTrajectoryOptimization.Subgraph, target: GcsTrajectoryOptimization.Subgraph, options: pydrake.geometry.optimization.GraphOfConvexSetsOptions = ...) -> tuple[pydrake.trajectories.CompositeTrajectory, pydrake.solvers.MathematicalProgramResult]: ...
    @staticmethod
    def UnwrapToContinuousTrajectory(gcs_trajectory: pydrake.trajectories.CompositeTrajectory, continuous_revolute_joints: list[int], starting_rounds: list[int] | None = ..., tol: float = ...) -> pydrake.trajectories.CompositeTrajectory: ...
    def continuous_revolute_joints(self) -> list[int]: ...
    def graph_of_convex_sets(self) -> pydrake.geometry.optimization.GraphOfConvexSets: ...
    def num_positions(self) -> int: ...

class IrisFromCliqueCoverOptions:
    coverage_termination_threshold: float
    iris_options: pydrake.geometry.optimization.IrisOptions | IrisNp2Options | IrisZoOptions
    iteration_limit: int
    minimum_clique_size: int
    num_points_per_coverage_check: int
    num_points_per_visibility_round: int
    parallelism: pydrake.common.Parallelism
    point_in_set_tol: float
    rank_tol_for_minimum_volume_circumscribed_ellipsoid: float
    def __init__(self) -> None: ...

class IrisNp2Options:
    add_hyperplane_if_solve_fails: bool
    parameterization: IrisParameterizationFunction
    ray_sampler_options: RaySamplerOptions
    sampled_iris_options: CommonSampledIrisOptions
    sampling_strategy: str
    solver: pydrake.solvers.SolverInterface
    solver_options: pydrake.solvers.SolverOptions
    def __init__(self) -> None: ...

class IrisParameterizationFunction:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, parameterization: Callable, dimension: int) -> None: ...
    @overload
    def __init__(self, expression_parameterization: numpy.ndarray[object[m, 1]], variables: numpy.ndarray[object[m, 1]]) -> None: ...
    @overload
    def __init__(self, kin: pydrake.multibody.rational.RationalForwardKinematics, q_star_val: numpy.ndarray[numpy.float64[m, 1]]) -> None: ...
    def get_parameterization_autodiff(self) -> Callable[[numpy.ndarray[object[m, 1]]], numpy.ndarray[object[m, 1]]]: ...
    def get_parameterization_dimension(self) -> int | None: ...
    def get_parameterization_double(self) -> Callable[[numpy.ndarray[numpy.float64[m, 1]]], numpy.ndarray[numpy.float64[m, 1]]]: ...
    def get_parameterization_is_threadsafe(self) -> bool: ...

class IrisZoOptions:
    bisection_steps: int
    parameterization: IrisParameterizationFunction
    sampled_iris_options: CommonSampledIrisOptions
    def __init__(self) -> None: ...

class JointLimits:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, plant: pydrake.multibody.plant.MultibodyPlant, require_finite_positions: bool = ..., require_finite_velocities: bool = ..., require_finite_accelerations: bool = ...) -> None: ...
    @overload
    def __init__(self, plant: pydrake.multibody.plant.MultibodyPlant, active_dof: DofMask, require_finite_positions: bool = ..., require_finite_velocities: bool = ..., require_finite_accelerations: bool = ...) -> None: ...
    @overload
    def __init__(self, other: JointLimits, active_dof: DofMask, require_finite_positions: bool = ..., require_finite_velocities: bool = ..., require_finite_accelerations: bool = ...) -> None: ...
    @overload
    def __init__(self, position_lower: numpy.ndarray[numpy.float64[m, 1]], position_upper: numpy.ndarray[numpy.float64[m, 1]], velocity_lower: numpy.ndarray[numpy.float64[m, 1]], velocity_upper: numpy.ndarray[numpy.float64[m, 1]], acceleration_lower: numpy.ndarray[numpy.float64[m, 1]], acceleration_upper: numpy.ndarray[numpy.float64[m, 1]], require_finite_positions: bool = ..., require_finite_velocities: bool = ..., require_finite_accelerations: bool = ...) -> None: ...
    def CheckInAccelerationLimits(self, acceleration: numpy.ndarray[numpy.float64[m, 1]], tolerance: float = ...) -> bool: ...
    def CheckInPositionLimits(self, position: numpy.ndarray[numpy.float64[m, 1]], tolerance: float = ...) -> bool: ...
    def CheckInVelocityLimits(self, velocity: numpy.ndarray[numpy.float64[m, 1]], tolerance: float = ...) -> bool: ...
    def acceleration_lower(self) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    def acceleration_upper(self) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    def num_accelerations(self) -> int: ...
    def num_positions(self) -> int: ...
    def num_velocities(self) -> int: ...
    def position_lower(self) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    def position_upper(self) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    def velocity_lower(self) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    def velocity_upper(self) -> numpy.ndarray[numpy.float64[m, 1]]: ...

class KinematicTrajectoryOptimization:
    @overload
    def __init__(self, num_positions: int, num_control_points: int, spline_order: int = ..., duration: float = ...) -> None: ...
    @overload
    def __init__(self, trajectory: pydrake.trajectories.BsplineTrajectory) -> None: ...
    def AddAccelerationBounds(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]]) -> list[pydrake.solvers.BindingTConstraintU]: ...
    def AddDurationConstraint(self, lb: float | None, ub: float | None) -> pydrake.solvers.BindingTBoundingBoxConstraintU: ...
    def AddDurationCost(self, weight: float = ...) -> pydrake.solvers.BindingTLinearCostU: ...
    def AddEffortBoundsAtNormalizedTimes(self, plant: pydrake.multibody.plant.MultibodyPlant, s: numpy.ndarray[numpy.float64[m, 1]], lb: numpy.ndarray[numpy.float64[m, 1]] | None = ..., ub: numpy.ndarray[numpy.float64[m, 1]] | None = ..., plant_context: pydrake.systems.framework.Context = ...) -> list[pydrake.solvers.BindingTConstraintU]: ...
    def AddJerkBounds(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]]) -> list[pydrake.solvers.BindingTConstraintU]: ...
    def AddPathAccelerationConstraint(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]], s: float) -> pydrake.solvers.BindingTLinearConstraintU: ...
    def AddPathEnergyCost(self, weight: float = ...) -> list[pydrake.solvers.BindingTCostU]: ...
    def AddPathLengthCost(self, weight: float = ..., use_conic_constraint: bool = ...) -> list[pydrake.solvers.BindingTCostU]: ...
    @overload
    def AddPathPositionConstraint(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]], s: float) -> pydrake.solvers.BindingTLinearConstraintU: ...
    @overload
    def AddPathPositionConstraint(self, constraint: pydrake.solvers.Constraint, s: float) -> pydrake.solvers.BindingTConstraintU: ...
    def AddPathVelocityConstraint(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]], s: float) -> pydrake.solvers.BindingTLinearConstraintU: ...
    def AddPositionBounds(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]]) -> list[pydrake.solvers.BindingTBoundingBoxConstraintU]: ...
    def AddVelocityBounds(self, lb: numpy.ndarray[numpy.float64[m, 1]], ub: numpy.ndarray[numpy.float64[m, 1]]) -> list[pydrake.solvers.BindingTLinearConstraintU]: ...
    @overload
    def AddVelocityConstraintAtNormalizedTime(self, constraint: pydrake.solvers.Constraint, s: float) -> pydrake.solvers.BindingTConstraintU: ...
    @overload
    def AddVelocityConstraintAtNormalizedTime(self, binding: pydrake.solvers.BindingTLinearConstraintU, s: float) -> list[pydrake.solvers.BindingTLinearConstraintU]: ...
    def ReconstructTrajectory(self, result: pydrake.solvers.MathematicalProgramResult) -> pydrake.trajectories.BsplineTrajectory: ...
    def SetInitialGuess(self, trajectory: pydrake.trajectories.BsplineTrajectory) -> None: ...
    def basis(self) -> pydrake.math.BsplineBasis: ...
    def control_points(self) -> numpy.ndarray[object[m, n]]: ...
    def duration(self) -> pydrake.symbolic.Variable: ...
    def get_mutable_prog(self) -> pydrake.solvers.MathematicalProgram: ...
    def num_control_points(self) -> int: ...
    def num_positions(self) -> int: ...
    def prog(self) -> pydrake.solvers.MathematicalProgram: ...
    def q(self) -> numpy.ndarray[object[m, 1]]: ...
    def qddot(self) -> numpy.ndarray[object[m, 1]]: ...
    def qdot(self) -> numpy.ndarray[object[m, 1]]: ...

class LinearDistanceAndInterpolationProvider(DistanceAndInterpolationProvider):
    @overload
    def __init__(self, plant: pydrake.multibody.plant.MultibodyPlant) -> None: ...
    @overload
    def __init__(self, plant: pydrake.multibody.plant.MultibodyPlant, distance_weights: numpy.ndarray[numpy.float64[m, 1]]) -> None: ...
    @overload
    def __init__(self, plant: pydrake.multibody.plant.MultibodyPlant, joint_distance_weights: dict[pydrake.multibody.tree.JointIndex, numpy.ndarray[numpy.float64[m, 1]]]) -> None: ...
    def distance_weights(self) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    def quaternion_dof_start_indices(self) -> list[int]: ...

class MaxCliqueSolverBase:
    def __init__(self) -> None: ...
    def SolveMaxClique(self, adjacency_matrix: scipy.sparse.csc_matrix[bool]) -> numpy.ndarray[bool[m, 1]]: ...

class MaxCliqueSolverViaGreedy(MaxCliqueSolverBase):
    def __init__(self) -> None: ...

class MaxCliqueSolverViaMip(MaxCliqueSolverBase):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, initial_guess: numpy.ndarray[numpy.float64[m, 1]] | None, solver_options: pydrake.solvers.SolverOptions) -> None: ...
    def GetInitialGuess(self) -> numpy.ndarray[numpy.float64[m, 1]] | None: ...
    def GetSolverOptions(self) -> pydrake.solvers.SolverOptions: ...
    def SetInitialGuess(self, initial_guess: numpy.ndarray[numpy.float64[m, 1]] | None) -> None: ...
    def SetSolverOptions(self, solver_options: pydrake.solvers.SolverOptions) -> None: ...

class MinCliqueCoverSolverBase:
    def __init__(self) -> None: ...
    def SolveMinCliqueCover(self, adjacency_matrix: scipy.sparse.csc_matrix[bool], partition: bool = ...) -> list[set[int]]: ...

class MinCliqueCoverSolverViaGreedy(MinCliqueCoverSolverBase):
    def __init__(self, max_clique_solver: MaxCliqueSolverBase, min_clique_size: int = ...) -> None: ...
    def get_min_clique_size(self) -> int: ...
    def set_min_clique_size(self, min_clique_size: int) -> None: ...

class MultipleShooting:
    def __init__(self, *args, **kwargs) -> None: ...
    def AddCompleteTrajectoryCallback(self, callback: Callable[[numpy.ndarray[numpy.float64[m, 1]], numpy.ndarray[numpy.float64[m, n], flags.f_contiguous], numpy.ndarray[numpy.float64[m, n], flags.f_contiguous], list[numpy.ndarray[numpy.float64[m, n], flags.f_contiguous]]], None], names: list[str]) -> pydrake.solvers.BindingTVisualizationCallbackU: ...
    @overload
    def AddConstraintToAllKnotPoints(self, f: pydrake.symbolic.Formula) -> list[pydrake.solvers.BindingTConstraintU]: ...
    @overload
    def AddConstraintToAllKnotPoints(self, f: numpy.ndarray[object[m, 1]]) -> list[pydrake.solvers.BindingTConstraintU]: ...
    @overload
    def AddConstraintToAllKnotPoints(self, constraint: pydrake.solvers.BoundingBoxConstraint, vars: numpy.ndarray[object[m, 1]]) -> list[pydrake.solvers.BindingTBoundingBoxConstraintU]: ...
    @overload
    def AddConstraintToAllKnotPoints(self, constraint: pydrake.solvers.LinearEqualityConstraint, vars: numpy.ndarray[object[m, 1]]) -> list[pydrake.solvers.BindingTLinearEqualityConstraintU]: ...
    @overload
    def AddConstraintToAllKnotPoints(self, constraint: pydrake.solvers.LinearConstraint, vars: numpy.ndarray[object[m, 1]]) -> list[pydrake.solvers.BindingTLinearConstraintU]: ...
    @overload
    def AddConstraintToAllKnotPoints(self, constraint: pydrake.solvers.Constraint, vars: numpy.ndarray[object[m, 1]]) -> list[pydrake.solvers.BindingTConstraintU]: ...
    def AddDurationBounds(self, lower_bound: float, upper_bound: float) -> pydrake.solvers.BindingTLinearConstraintU: ...
    def AddEqualTimeIntervalsConstraints(self) -> list[pydrake.solvers.BindingTLinearConstraintU]: ...
    @overload
    def AddFinalCost(self, e: pydrake.symbolic.Expression) -> pydrake.solvers.BindingTCostU: ...
    @overload
    def AddFinalCost(self, matrix: numpy.ndarray[object[m, n], flags.f_contiguous]) -> pydrake.solvers.BindingTCostU: ...
    def AddInputTrajectoryCallback(self, callback: Callable[[numpy.ndarray[numpy.float64[m, 1]], numpy.ndarray[numpy.float64[m, n], flags.f_contiguous]], None]) -> pydrake.solvers.BindingTVisualizationCallbackU: ...
    def AddRunningCost(self, g: pydrake.symbolic.Expression) -> None: ...
    def AddStateTrajectoryCallback(self, callback: Callable[[numpy.ndarray[numpy.float64[m, 1]], numpy.ndarray[numpy.float64[m, n], flags.f_contiguous]], None]) -> pydrake.solvers.BindingTVisualizationCallbackU: ...
    def AddTimeIntervalBounds(self, lower_bound: float, upper_bound: float) -> pydrake.solvers.BindingTBoundingBoxConstraintU: ...
    def GetInputSamples(self, result: pydrake.solvers.MathematicalProgramResult) -> numpy.ndarray[numpy.float64[m, n]]: ...
    def GetSampleTimes(self, result: pydrake.solvers.MathematicalProgramResult) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    def GetSequentialVariableAtIndex(self, name: str, index: int) -> numpy.ndarray[object[m, 1]]: ...
    def GetSequentialVariableSamples(self, result: pydrake.solvers.MathematicalProgramResult, name: str) -> numpy.ndarray[numpy.float64[m, n]]: ...
    def GetStateSamples(self, result: pydrake.solvers.MathematicalProgramResult) -> numpy.ndarray[numpy.float64[m, n]]: ...
    def NewSequentialVariable(self, rows: int, name: str) -> numpy.ndarray[object[m, 1]]: ...
    def ReconstructInputTrajectory(self, result: pydrake.solvers.MathematicalProgramResult) -> pydrake.trajectories.PiecewisePolynomial: ...
    def ReconstructStateTrajectory(self, result: pydrake.solvers.MathematicalProgramResult) -> pydrake.trajectories.PiecewisePolynomial: ...
    def SetInitialTrajectory(self, traj_init_u: pydrake.trajectories.PiecewisePolynomial, traj_init_x: pydrake.trajectories.PiecewisePolynomial) -> None: ...
    def final_state(self) -> numpy.ndarray[object[m, 1]]: ...
    def fixed_time_step(self) -> float: ...
    def initial_state(self) -> numpy.ndarray[object[m, 1]]: ...
    @overload
    def input(self) -> numpy.ndarray[object[m, 1]]: ...
    @overload
    def input(self, index: int) -> numpy.ndarray[object[m, 1]]: ...
    def prog(self) -> pydrake.solvers.MathematicalProgram: ...
    @overload
    def state(self) -> numpy.ndarray[object[m, 1]]: ...
    @overload
    def state(self, index: int) -> numpy.ndarray[object[m, 1]]: ...
    def time(self) -> numpy.ndarray[object[1, 1]]: ...
    def time_step(self, index: int) -> numpy.ndarray[object[1, 1]]: ...

class RaySamplerOptions:
    num_particles_to_walk_towards: int
    only_walk_toward_collisions: bool
    ray_search_num_steps: int
    def __init__(self) -> None: ...

class RobotClearance:
    @overload
    def __init__(self, num_positions: int) -> None: ...
    @overload
    def __init__(self, other: RobotClearance) -> None: ...
    def Append(self, robot_index: pydrake.multibody.tree.BodyIndex, other_index: pydrake.multibody.tree.BodyIndex, collision_type: RobotCollisionType, distance: float, jacobian: numpy.ndarray[numpy.float64[1, n]]) -> None: ...
    def Reserve(self, size: int) -> None: ...
    def collision_types(self) -> list[RobotCollisionType]: ...
    def distances(self) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    def jacobians(self) -> numpy.ndarray[numpy.float64[m, n], flags.c_contiguous]: ...
    def mutable_jacobians(self) -> numpy.ndarray[numpy.float64[m, n], flags.writeable, flags.c_contiguous]: ...
    def num_positions(self) -> int: ...
    def other_indices(self) -> list[pydrake.multibody.tree.BodyIndex]: ...
    def robot_indices(self) -> list[pydrake.multibody.tree.BodyIndex]: ...
    def size(self) -> int: ...
    def __copy__(self) -> RobotClearance: ...
    def __deepcopy__(self, arg0: dict) -> RobotClearance: ...

class RobotCollisionType:
    __members__: ClassVar[dict] = ...  # read-only
    __entries: ClassVar[dict] = ...
    kEnvironmentAndSelfCollision: ClassVar[RobotCollisionType] = ...
    kEnvironmentCollision: ClassVar[RobotCollisionType] = ...
    kNoCollision: ClassVar[RobotCollisionType] = ...
    kSelfCollision: ClassVar[RobotCollisionType] = ...
    def __init__(self, value: int) -> None: ...
    def MakeUpdated(self, in_environment_collision: bool | None = ..., in_self_collision: bool | None = ...) -> RobotCollisionType: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class RobotDiagram(pydrake.systems.framework.Diagram):
    def __init__(self, *args, **kwargs) -> None: ...
    def mutable_plant_context(self, root_context: pydrake.systems.framework.Context) -> pydrake.systems.framework.Context: ...
    def mutable_scene_graph(self) -> pydrake.geometry.SceneGraph: ...
    def mutable_scene_graph_context(self, root_context: pydrake.systems.framework.Context) -> pydrake.systems.framework.Context: ...
    @overload
    def plant(self) -> pydrake.multibody.plant.MultibodyPlant: ...
    @overload
    def plant(self, readonly) -> Any: ...
    def plant_context(self, root_context: pydrake.systems.framework.Context) -> pydrake.systems.framework.Context: ...
    def scene_graph(self) -> pydrake.geometry.SceneGraph: ...
    def scene_graph_context(self, root_context: pydrake.systems.framework.Context) -> pydrake.systems.framework.Context: ...

class RobotDiagramBuilder:
    def __init__(self, time_step: float = ...) -> None: ...
    def Build(self) -> object: ...
    def IsDiagramBuilt(self) -> bool: ...
    def builder(self) -> pydrake.systems.framework.DiagramBuilder: ...
    def parser(self) -> pydrake.multibody.parsing.Parser: ...
    @overload
    def plant(self) -> pydrake.multibody.plant.MultibodyPlant: ...
    @overload
    def plant(self, mutable) -> Any: ...
    def scene_graph(self) -> pydrake.geometry.SceneGraph: ...

class RobotDiagramBuilder_𝓣AutoDiffXd𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, time_step: float = ...) -> None: ...
    def Build(self) -> object: ...
    def IsDiagramBuilt(self) -> bool: ...
    def builder(self) -> pydrake.systems.framework.DiagramBuilder_TAutoDiffXdU: ...
    @overload
    def plant(self) -> pydrake.multibody.plant.MultibodyPlant_TAutoDiffXdU: ...
    @overload
    def plant(self, mutable) -> Any: ...
    def scene_graph(self) -> pydrake.geometry.SceneGraph_TAutoDiffXdU: ...

class RobotDiagramBuilder_𝓣Expression𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, time_step: float = ...) -> None: ...
    def Build(self) -> object: ...
    def IsDiagramBuilt(self) -> bool: ...
    def builder(self) -> pydrake.systems.framework.DiagramBuilder_TExpressionU: ...
    @overload
    def plant(self) -> pydrake.multibody.plant.MultibodyPlant_TExpressionU: ...
    @overload
    def plant(self, mutable) -> Any: ...
    def scene_graph(self) -> pydrake.geometry.SceneGraph_TExpressionU: ...

class RobotDiagram_𝓣AutoDiffXd𝓤(pydrake.systems.framework.Diagram_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def mutable_plant_context(self, root_context: pydrake.systems.framework.Context_TAutoDiffXdU) -> pydrake.systems.framework.Context_TAutoDiffXdU: ...
    def mutable_scene_graph(self) -> pydrake.geometry.SceneGraph_TAutoDiffXdU: ...
    def mutable_scene_graph_context(self, root_context: pydrake.systems.framework.Context_TAutoDiffXdU) -> pydrake.systems.framework.Context_TAutoDiffXdU: ...
    @overload
    def plant(self) -> pydrake.multibody.plant.MultibodyPlant_TAutoDiffXdU: ...
    @overload
    def plant(self, readonly) -> Any: ...
    def plant_context(self, root_context: pydrake.systems.framework.Context_TAutoDiffXdU) -> pydrake.systems.framework.Context_TAutoDiffXdU: ...
    def scene_graph(self) -> pydrake.geometry.SceneGraph_TAutoDiffXdU: ...
    def scene_graph_context(self, root_context: pydrake.systems.framework.Context_TAutoDiffXdU) -> pydrake.systems.framework.Context_TAutoDiffXdU: ...

class RobotDiagram_𝓣Expression𝓤(pydrake.systems.framework.Diagram_TExpressionU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def mutable_plant_context(self, root_context: pydrake.systems.framework.Context_TExpressionU) -> pydrake.systems.framework.Context_TExpressionU: ...
    def mutable_scene_graph(self) -> pydrake.geometry.SceneGraph_TExpressionU: ...
    def mutable_scene_graph_context(self, root_context: pydrake.systems.framework.Context_TExpressionU) -> pydrake.systems.framework.Context_TExpressionU: ...
    @overload
    def plant(self) -> pydrake.multibody.plant.MultibodyPlant_TExpressionU: ...
    @overload
    def plant(self, readonly) -> Any: ...
    def plant_context(self, root_context: pydrake.systems.framework.Context_TExpressionU) -> pydrake.systems.framework.Context_TExpressionU: ...
    def scene_graph(self) -> pydrake.geometry.SceneGraph_TExpressionU: ...
    def scene_graph_context(self, root_context: pydrake.systems.framework.Context_TExpressionU) -> pydrake.systems.framework.Context_TExpressionU: ...

class SceneGraphCollisionChecker(CollisionChecker):
    def __init__(self, params: CollisionCheckerParams) -> None: ...

class UnimplementedCollisionChecker(CollisionChecker):
    def __init__(self, params: CollisionCheckerParams, supports_parallel_checking: bool) -> None: ...

class ZmpPlanner:
    def __init__(self) -> None: ...
    def ComputeOptimalCoMdd(self, time: float, x: numpy.ndarray[numpy.float64[4, 1]]) -> numpy.ndarray[numpy.float64[2, 1]]: ...
    def Plan(self, zmp_d: pydrake.trajectories.PiecewisePolynomial, x0: numpy.ndarray[numpy.float64[4, 1]], height: float, gravity: float = ..., Qy: numpy.ndarray[numpy.float64[2, 2]] = ..., R: numpy.ndarray[numpy.float64[2, 2]] = ...) -> None: ...
    def comdd_to_cop(self, x: numpy.ndarray[numpy.float64[4, 1]], u: numpy.ndarray[numpy.float64[2, 1]]) -> numpy.ndarray[numpy.float64[2, 1]]: ...
    def get_A(self) -> numpy.ndarray[numpy.float64[4, 4]]: ...
    def get_B(self) -> numpy.ndarray[numpy.float64[4, 2]]: ...
    def get_C(self) -> numpy.ndarray[numpy.float64[2, 4]]: ...
    def get_D(self) -> numpy.ndarray[numpy.float64[2, 2]]: ...
    def get_Qy(self) -> numpy.ndarray[numpy.float64[2, 2]]: ...
    def get_R(self) -> numpy.ndarray[numpy.float64[2, 2]]: ...
    @overload
    def get_Vx(self, time: float) -> numpy.ndarray[numpy.float64[4, 1]]: ...
    @overload
    def get_Vx(self) -> pydrake.trajectories.ExponentialPlusPiecewisePolynomial: ...
    def get_Vxx(self) -> numpy.ndarray[numpy.float64[4, 4]]: ...
    @overload
    def get_desired_zmp(self, time: float) -> numpy.ndarray[numpy.float64[2, 1]]: ...
    @overload
    def get_desired_zmp(self) -> pydrake.trajectories.PiecewisePolynomial: ...
    def get_final_desired_zmp(self) -> numpy.ndarray[numpy.float64[2, 1]]: ...
    @overload
    def get_nominal_com(self, time: float) -> numpy.ndarray[numpy.float64[2, 1]]: ...
    @overload
    def get_nominal_com(self) -> pydrake.trajectories.ExponentialPlusPiecewisePolynomial: ...
    @overload
    def get_nominal_comd(self, time: float) -> numpy.ndarray[numpy.float64[2, 1]]: ...
    @overload
    def get_nominal_comd(self) -> pydrake.trajectories.ExponentialPlusPiecewisePolynomial: ...
    @overload
    def get_nominal_comdd(self, time: float) -> numpy.ndarray[numpy.float64[2, 1]]: ...
    @overload
    def get_nominal_comdd(self) -> pydrake.trajectories.ExponentialPlusPiecewisePolynomial: ...
    def has_planned(self) -> bool: ...
    def __copy__(self) -> ZmpPlanner: ...
    def __deepcopy__(self, arg0: dict) -> ZmpPlanner: ...

class _TemporaryName_N5drake8planning12RobotDiagramINS_2ad8AutoDiffEEE(pydrake.systems.framework.Diagram_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def mutable_plant_context(self, root_context: pydrake.systems.framework.Context_TAutoDiffXdU) -> pydrake.systems.framework.Context_TAutoDiffXdU: ...
    def mutable_scene_graph(self) -> pydrake.geometry.SceneGraph_TAutoDiffXdU: ...
    def mutable_scene_graph_context(self, root_context: pydrake.systems.framework.Context_TAutoDiffXdU) -> pydrake.systems.framework.Context_TAutoDiffXdU: ...
    @overload
    def plant(self) -> pydrake.multibody.plant.MultibodyPlant_TAutoDiffXdU: ...
    @overload
    def plant(self, readonly) -> Any: ...
    def plant_context(self, root_context: pydrake.systems.framework.Context_TAutoDiffXdU) -> pydrake.systems.framework.Context_TAutoDiffXdU: ...
    def scene_graph(self) -> pydrake.geometry.SceneGraph_TAutoDiffXdU: ...
    def scene_graph_context(self, root_context: pydrake.systems.framework.Context_TAutoDiffXdU) -> pydrake.systems.framework.Context_TAutoDiffXdU: ...

class _TemporaryName_N5drake8planning12RobotDiagramINS_8symbolic10ExpressionEEE(pydrake.systems.framework.Diagram_TExpressionU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def mutable_plant_context(self, root_context: pydrake.systems.framework.Context_TExpressionU) -> pydrake.systems.framework.Context_TExpressionU: ...
    def mutable_scene_graph(self) -> pydrake.geometry.SceneGraph_TExpressionU: ...
    def mutable_scene_graph_context(self, root_context: pydrake.systems.framework.Context_TExpressionU) -> pydrake.systems.framework.Context_TExpressionU: ...
    @overload
    def plant(self) -> pydrake.multibody.plant.MultibodyPlant_TExpressionU: ...
    @overload
    def plant(self, readonly) -> Any: ...
    def plant_context(self, root_context: pydrake.systems.framework.Context_TExpressionU) -> pydrake.systems.framework.Context_TExpressionU: ...
    def scene_graph(self) -> pydrake.geometry.SceneGraph_TExpressionU: ...
    def scene_graph_context(self, root_context: pydrake.systems.framework.Context_TExpressionU) -> pydrake.systems.framework.Context_TExpressionU: ...

class _TemporaryName_N5drake8planning19RobotDiagramBuilderINS_2ad8AutoDiffEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, time_step: float = ...) -> None: ...
    def Build(self) -> object: ...
    def IsDiagramBuilt(self) -> bool: ...
    def builder(self) -> pydrake.systems.framework.DiagramBuilder_TAutoDiffXdU: ...
    @overload
    def plant(self) -> pydrake.multibody.plant.MultibodyPlant_TAutoDiffXdU: ...
    @overload
    def plant(self, mutable) -> Any: ...
    def scene_graph(self) -> pydrake.geometry.SceneGraph_TAutoDiffXdU: ...

class _TemporaryName_N5drake8planning19RobotDiagramBuilderINS_8symbolic10ExpressionEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, time_step: float = ...) -> None: ...
    def Build(self) -> object: ...
    def IsDiagramBuilt(self) -> bool: ...
    def builder(self) -> pydrake.systems.framework.DiagramBuilder_TExpressionU: ...
    @overload
    def plant(self) -> pydrake.multibody.plant.MultibodyPlant_TExpressionU: ...
    @overload
    def plant(self, mutable) -> Any: ...
    def scene_graph(self) -> pydrake.geometry.SceneGraph_TExpressionU: ...

def AddDirectCollocationConstraint(constraint: DirectCollocationConstraint, time_step: numpy.ndarray[object[m, 1]], state: numpy.ndarray[object[m, 1]], next_state: numpy.ndarray[object[m, 1]], input: numpy.ndarray[object[m, 1]], next_input: numpy.ndarray[object[m, 1]], prog: pydrake.solvers.MathematicalProgram) -> pydrake.solvers.BindingTConstraintU: ...
def GetContinuousRevoluteJointIndices(plant: pydrake.multibody.plant.MultibodyPlant) -> list[int]: ...
def IrisInConfigurationSpaceFromCliqueCover(checker: CollisionChecker, options: IrisFromCliqueCoverOptions, generator: pydrake.common.RandomGenerator, sets: list[pydrake.geometry.optimization.HPolyhedron], max_clique_solver: MaxCliqueSolverBase = ...) -> list[pydrake.geometry.optimization.HPolyhedron]: ...
def IrisNp2(checker: SceneGraphCollisionChecker, starting_ellipsoid: pydrake.geometry.optimization.Hyperellipsoid, domain: pydrake.geometry.optimization.HPolyhedron, options: IrisNp2Options = ...) -> pydrake.geometry.optimization.HPolyhedron: ...
def IrisZo(checker: CollisionChecker, starting_ellipsoid: pydrake.geometry.optimization.Hyperellipsoid, domain: pydrake.geometry.optimization.HPolyhedron, options: IrisZoOptions = ...) -> pydrake.geometry.optimization.HPolyhedron: ...
def MakeBodyShapeDescription(plant: pydrake.multibody.plant.MultibodyPlant, plant_context: pydrake.systems.framework.Context, geometry_id: pydrake.geometry.GeometryId) -> BodyShapeDescription: ...
def VisibilityGraph(checker: CollisionChecker, points: numpy.ndarray[numpy.float64[m, n], flags.f_contiguous], parallelize: pydrake.common.Parallelism = ...) -> scipy.sparse.csc_matrix[bool]: ...



"""
Provides support for using ipywidgets with the systems framework, and a number
of useful widget systems.

This is gui code; to test changes, please manually execute:

    bazel run //bindings/pydrake/systems:jupyter_widgets_examples
"""

from collections import namedtuple
from functools import partial

from IPython.display import display
from ipywidgets import FloatSlider, Layout, Widget
import numpy as np

from pydrake.common.jupyter import process_ipywidget_events
from pydrake.common.value import AbstractValue
from pydrake.math import RigidTransform, RollPitchYaw
from pydrake.systems.framework import LeafSystem, PublishEvent


class PoseSliders(LeafSystem):
    """
    Provides a set of ipywidget sliders (to be used in a Jupyter notebook) with
    one slider for each of roll, pitch, yaw, x, y, and z.  This can be used,
    for instance, as an interface to teleoperate the end-effector of a robot.

    .. pydrake_system::

        name: PoseSliders
        output_ports:
        - pose
    """

    # TODO(russt): Use namedtuple defaults parameter once we are Python >= 3.7.
    Visible = namedtuple("Visible", ("roll", "pitch", "yaw", "x", "y", "z"))
    Visible.__new__.__defaults__ = (True, True, True, True, True, True)
    MinRange = namedtuple("MinRange", ("roll", "pitch", "yaw", "x", "y", "z"))
    MinRange.__new__.__defaults__ = (-np.pi, -np.pi, -np.pi, -1.0, -1.0, -1.0)
    MaxRange = namedtuple("MaxRange", ("roll", "pitch", "yaw", "x", "y", "z"))
    MaxRange.__new__.__defaults__ = (np.pi, np.pi, np.pi, 1.0, 1.0, 1.0)
    Value = namedtuple("Value", ("roll", "pitch", "yaw", "x", "y", "z"))
    Value.__new__.__defaults__ = (0.0, 0.0, 0.0, 0.0, 0.0, 0.0)

    def __init__(
        self,
        visible=Visible(),
        min_range=MinRange(),
        max_range=MaxRange(),
        value=Value(),
    ):
        """
        Args:
            visible: An object with boolean elements for 'roll', 'pitch',
                     'yaw', 'x', 'y', 'z'; the intention is for this to be the
                     PoseSliders.Visible() namedtuple.  Defaults to all true.
            min_range, max_range, value: Objects with float values for 'roll',
                      'pitch', 'yaw', 'x', 'y', 'z'; the intention is for the
                      caller to use the PoseSliders.MinRange, MaxRange, and
                      Value namedtuples.  See those tuples for default values.
        """
        LeafSystem.__init__(self)
        port = self.DeclareAbstractOutputPort(
            "pose",
            lambda: AbstractValue.Make(RigidTransform()),
            self.DoCalcOutput,
        )

        # The widgets themselves have undeclared state.  For now, we accept it,
        # and simply disable caching on the output port.
        # TODO(russt): consider implementing the more elaborate methods seen
        # in, e.g., LcmMessageSubscriber.
        port.disable_caching_by_default()

        # Note: This timing affects the keyboard teleop performance. A larger
        #       time step causes more lag in the response.
        self.DeclarePeriodicEvent(
            0.01, 0.0, PublishEvent(self._process_event_queue)
        )

        self._roll = FloatSlider(
            min=min_range.roll,
            max=max_range.roll,
            value=value.roll,
            step=0.01,
            continuous_update=True,
            description="roll",
            layout=Layout(width="90%"),
        )
        self._pitch = FloatSlider(
            min=min_range.pitch,
            max=max_range.pitch,
            value=value.pitch,
            step=0.01,
            continuous_update=True,
            description="pitch",
            layout=Layout(width="90%"),
        )
        self._yaw = FloatSlider(
            min=min_range.yaw,
            max=max_range.yaw,
            value=value.yaw,
            step=0.01,
            continuous_update=True,
            description="yaw",
            layout=Layout(width="90%"),
        )
        self._x = FloatSlider(
            min=min_range.x,
            max=max_range.x,
            value=value.x,
            step=0.01,
            continuous_update=True,
            description="x",
            orient="horizontal",
            layout=Layout(width="90%"),
        )
        self._y = FloatSlider(
            min=min_range.y,
            max=max_range.y,
            value=value.y,
            step=0.01,
            continuous_update=True,
            description="y",
            layout=Layout(width="90%"),
        )
        self._z = FloatSlider(
            min=min_range.z,
            max=max_range.z,
            value=value.z,
            step=0.01,
            continuous_update=True,
            description="z",
            layout=Layout(width="90%"),
        )

        if visible.roll:
            display(self._roll)
        if visible.pitch:
            display(self._pitch)
        if visible.yaw:
            display(self._yaw)
        if visible.x:
            display(self._x)
        if visible.y:
            display(self._y)
        if visible.z:
            display(self._z)

    def SetPose(self, pose):
        """
        Sets the current value of the sliders.
        Args:
            pose: Any viable argument for the RigidTransform
                  constructor.
        """
        tf = RigidTransform(pose)
        self.SetRpy(RollPitchYaw(tf.rotation()))
        self.SetXyz(tf.translation())

    def SetRpy(self, rpy):
        """
        Sets the current value of the sliders for roll, pitch, and yaw.
        Args:
            rpy: An instance of drake.math.RollPitchYaw
        """
        self._roll.value = rpy.roll_angle()
        self._pitch.value = rpy.pitch_angle()
        self._yaw.value = rpy.yaw_angle()

    def SetXyz(self, xyz):
        """
        Sets the current value of the sliders for x, y, and z.
        Args:
            xyz: A 3 element iterable object with x, y, z.
        """
        self._x.value = xyz[0]
        self._y.value = xyz[1]
        self._z.value = xyz[2]

    def _process_event_queue(self, context, event):
        """
        Allows the ipython kernel to process the event queue.
        """
        process_ipywidget_events()

    def DoCalcOutput(self, context, output):
        """
        Constructs the output values from the widget elements.
        """
        output.set_value(
            RigidTransform(
                RollPitchYaw(
                    self._roll.value, self._pitch.value, self._yaw.value
                ),
                [self._x.value, self._y.value, self._z.value],
            )
        )


class WidgetSystem(LeafSystem):
    """
    A system that outputs the ``value``s (converted to ``float``) from
    ipywidgets.

    .. pydrake_system::

      name: WidgetSystem
      output_ports:
      - widget_group_0
      - ...
      - widget_group_{N-1}
    """

    def __init__(self, *args, update_period_sec=0.1):
        """
        Constructs the system and displays the widgets.

        Args:
            update_period_sec: Specifies how often the kernel is queried for
                widget ui events.

        An output port is created for each element in *args.  Each element of
        args must itself be an iterable collection (list, tuple, set, ...) of
        ipywidget elements, whose values will be concatenated into a single
        vector output.
        """
        LeafSystem.__init__(self)

        self._widgets = args
        for i, widget_iterable in enumerate(self._widgets):
            for w in widget_iterable:
                assert isinstance(w, Widget), (
                    "args must be collections of widgets"
                )
                display(w)
            port = self.DeclareVectorOutputPort(
                f"widget_group_{i}",
                len(widget_iterable),
                partial(self.DoCalcOutput, port_index=i),
            )
            port.disable_caching_by_default()

        self.DeclarePeriodicEvent(
            update_period_sec, 0.0, PublishEvent(self._process_event_queue)
        )

    def _process_event_queue(self, unused_context, unused_event):
        """
        Allows the ipython kernel to process the event queue.
        """
        process_ipywidget_events()

    def DoCalcOutput(self, context, output, port_index):
        """
        Constructs the output values from the widget elements.
        """
        for i, w in enumerate(self._widgets[port_index]):
            output.SetAtIndex(i, float(w.value))



"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

class lcmt_scope(object):
    """ The message type for LcmScopeSystem. """

    __slots__ = ["utime", "size", "value"]

    __typenames__ = ["int64_t", "int32_t", "double"]

    __dimensions__ = [None, None, ["size"]]

    def __init__(self):
        self.utime = 0
        """
        The timestamp in microseconds.
        LCM Type: int64_t
        """

        self.size = 0
        """
        The scoped value.
        LCM Type: int32_t
        """

        self.value = []
        """ LCM Type: double[size] """

    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_scope._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        buf.write(struct.pack(">qi", self.utime, self.size))
        buf.write(struct.pack('>%dd' % self.size, *self.value[:self.size]))

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_scope._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_scope._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_scope()
        self.utime, self.size = struct.unpack(">qi", buf.read(12))
        self.value = struct.unpack('>%dd' % self.size, buf.read(self.size * 8))
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_scope in parents: return 0
        tmphash = (0x4806c9b395a3fbef) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_scope._packed_fingerprint is None:
            lcmt_scope._packed_fingerprint = struct.pack(">Q", lcmt_scope._get_hash_recursive([]))
        return lcmt_scope._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_scope._get_packed_fingerprint())[0]




import numpy as _np


def _scalar_types_match(actual_type, expected_type):
    return (actual_type is expected_type) or (
        (expected_type is float) and (actual_type in (_np.float32, _np.float64))
    )


def _wrap_user_evaluator_func(
    cls_name: str,
    user_evaluator_func,
    num_vars: int,
    num_outputs: int,
    output_dim: int,
    expected_type: type,
):
    """The implementation of the C++ helper `WrapUserEvaluatorFunc`."""
    assert output_dim in {0, 1}
    if output_dim == 0:
        assert num_outputs == 1

    def _wrapped(x):
        # This assertion is guaranteed by the C++ signature that calls us.
        assert x.ndim == 1

        # Check that the user passed the correct number of variables to the
        # evaluator. When called from MathematicalProgram this is guaranteed,
        # but direct calls to the evaluator might still have a mismatch.
        #
        # N.B. The C++ Eval code that calls us (via DoEval) in will (in Debug
        # builds) assertion-check the size, so this Python check is only
        # reachable in Release builds.
        if x.size != num_vars:
            raise ValueError(
                f"{cls_name} input must have .size = {num_vars}. "
                f"Got .size = {x.size} instead."
            )

        # Call the user's evaluator.
        y = user_evaluator_func(x)

        # For costs, the return value should be a scalar of the expected type.
        if output_dim == 0:
            if not _scalar_types_match(type(y), expected_type):
                raise TypeError(
                    f"When {cls_name} is called with an array of type "
                    f"{expected_type.__name__} the return value must be a "
                    "scalar (not array) of the same type, not a "
                    f"{type(y).__name__} ({y!r})."
                )
            return y

        # For constraints, the return value should be a vector of the expected
        # type (either an np.ndarray or convertible to it).
        if y is None:
            raise TypeError(f"{cls_name} returned None")
        try:
            y = _np.asarray(y)
        except Exception as e:
            raise TypeError(
                f"When {cls_name} is called with an array of type "
                f"{expected_type.__name__} the return value must be an array "
                f"of the same type (numpy conversion error: {e})."
            )

        # Validate the shape.
        valid = (y.size == num_outputs) and (
            y.ndim == 1 or (y.ndim == 2 and 1 in y.shape)
        )
        if not valid:
            raise ValueError(
                f"{cls_name} return value must be array of "
                f".ndim = 1 or 2 (vector) and .size = {num_outputs}. "
                f"Got .ndim = {y.ndim} and .size = {y.size} instead."
            )
        if num_outputs == 0:
            return y

        # Validate the dtype.
        if y.dtype != object:
            if y.dtype is float or y.dtype is int:
                actual_type = float
            else:
                actual_type = y.dtype.type
        else:
            # Search array for any non-builtin and non-numpy types.
            for yi in y.flat:
                t = type(yi)
                if t.__module__ not in ("builtins", "numpy"):
                    actual_type = t
                    break
            else:
                # Unable to infer type.
                actual_type = type(None)
        if not _scalar_types_match(actual_type, expected_type):
            raise TypeError(
                f"When {cls_name} is called with an array of type "
                f"{expected_type.__name__} the return value must be the same "
                f"type, not {actual_type.__name__}."
            )

        return y

    return _wrapped



"""Provides extensions for containers of Drake-related objects."""

import re

import numpy as np


class _EqualityProxyBase:
    # Wraps an object with a non-compliant `__eq__` operator (returns a
    # non-bool convertible expression) with a custom compliant `__eq__`
    # operator.
    def __init__(self, value):
        self._value = value

    def _get_value(self):
        return self._value

    def __hash__(self):
        return hash(self._value)

    def __eq__(self, other):
        raise NotImplementedError("Abstract method")

    value = property(_get_value)


class _DictKeyWrap(dict):
    # Wraps a dictionary's key access. For a key of a type `TOrig`, this
    # dictionary will provide a key of type `TProxy`, that should proxy the
    # original key.
    def __init__(self, dict_in, key_wrap, key_unwrap):
        # @param dict_in Dictionary with keys of types TOrig (not necessarily
        # homogeneous).
        # @param key_wrap Functor that maps from TOrig -> TProxy.
        # @param key_unwrap Functor that maps from TProxy -> TOrig.
        dict.__init__(self)
        # N.B. Passing properties to these will cause an issue. This can be
        # sidestepped by storing the properties in a `dict`.
        self._key_wrap = key_wrap
        self._key_unwrap = key_unwrap
        for key, value in dict_in.items():
            self[key] = value

    def __setitem__(self, key, value):
        return dict.__setitem__(self, self._key_wrap(key), value)

    def __getitem__(self, key):
        return dict.__getitem__(self, self._key_wrap(key))

    def __delitem__(self, key):
        return dict.__delitem__(self, self._key_wrap(key))

    def __contains__(self, key):
        return dict.__contains__(self, self._key_wrap(key))

    def items(self):
        return zip(self.keys(), self.values())

    def keys(self):
        return (self._key_unwrap(key) for key in dict.keys(self))

    def raw(self):
        """Returns a dict with the original keys.

        Note:
            Copying to a `dict` will maintain the proxy keys.
        """
        return dict(self.items())


class EqualToDict(_DictKeyWrap):
    """Implements a dictionary where keys are compared using type and
    `lhs.EqualTo(rhs)`.
    """

    def __init__(self, *args, **kwargs):
        class Proxy(_EqualityProxyBase):
            def __eq__(self, other):
                T = type(self.value)
                return isinstance(other.value, T) and self.value.EqualTo(
                    other.value
                )

            # https://stackoverflow.com/a/1608907/7829525
            __hash__ = _EqualityProxyBase.__hash__

        dict_in = dict(*args, **kwargs)
        _DictKeyWrap.__init__(self, dict_in, Proxy, Proxy._get_value)


class NamedViewBase:
    """Base for classes generated by ``namedview``.

    Inspired by: https://gitlab.com/ericvsmith/namedlist
    """

    _fields = None  # To be specified by inherited classes.

    def __init__(self, value):
        """Creates a view on ``value``. Any mutations on this instance will be
        reflected in ``value``, and any mutations on ``value`` will be
        reflected in this instance."""
        assert self._fields is not None, (
            "Class must be generated by ``namedview``"
        )
        assert len(self._fields) == len(value)
        object.__setattr__(self, "_value", value)

    @classmethod
    def get_fields(cls):
        """Returns all fields for this class or object."""
        return cls._fields

    def __getitem__(self, i):
        return self._value[i]

    def __setitem__(self, i, value_i):
        self._value[i] = value_i

    def __setattr__(self, name, value):
        """Prevent setting additional attributes."""
        if not hasattr(self, name):
            raise AttributeError(
                "Cannot add attributes! The fields in this named view are"
                f"{self.get_fields()}, but you tried to set '{name}'."
            )
        object.__setattr__(self, name, value)

    def __len__(self):
        return len(self._value)

    def __iter__(self):
        return iter(self._value)

    def __array__(self):
        """Proxy for use with NumPy."""
        return np.asarray(self._value)

    def _str_like(self, *, per_field_op):
        """Provides human-readable breakout of each field and value."""
        value_strs = []
        for i, field in enumerate(self._fields):
            value_strs.append("{}={}".format(field, per_field_op(self[i])))
        return "{}({})".format(self.__class__.__name__, ", ".join(value_strs))

    def __str__(self):
        return self._str_like(per_field_op=str)

    def __repr__(self):
        inner_text = self._str_like(per_field_op=repr)
        return f"<{inner_text}>"

    @staticmethod
    def _item_property(i):
        # Maps an item (at a given index) to a property.
        return property(
            fget=lambda self: self[i],
            fset=lambda self, value: self.__setitem__(i, value),
        )

    @classmethod
    def Zero(cls):
        """Constructs a view onto values set to all zeros."""
        return cls([0] * len(cls._fields))


def _sanitize_field_name(name: str):
    result = name
    # Ensure the first character is a valid opener (e.g., no numbers allowed).
    if not result[0].isidentifier():
        result = "_" + result
    # Ensure that each additional character is valid in turn, avoiding the
    # special case for opening characters by prepending "_" during the check.
    for i in range(1, len(result)):
        if not ("_" + result[i]).isidentifier():
            result = result[:i] + "_" + result[i + 1 :]
    result = re.sub("__+", "_", result)
    assert result.isidentifier(), f"Sanitization failed on {name} => {result}"
    return result


def namedview(name, fields, *, sanitize_field_names=True):
    """
    Creates a class that is a named view with given ``fields``. When the class
    is instantiated, it must be given the object that it will be a proxy for.
    Similar to ``namedtuple``.

    If ``sanitize_field_names`` is True (the default), then any characters in
    ``fields`` which are not valid in Python identifiers will be automatically
    replaced with `_`. Leading numbers will have `_` inserted, and duplicate
    `_` will be replaced by a single `_`.

    Example:
        ::

            MyView = namedview("MyView", ('a', 'b'))

            value = np.array([1, 2])
            view = MyView(value)
            view.a = 10  # `value` is now [10, 2]
            value[1] = 100  # `view` is now [10, 100]
            view[:] = 3  # `value` is now [3, 3]

            # Get an array from the view *aliasing* the original vector.
            value_view = view[:]
            # Another way to get an aliased array.
            value_view_2 = np.asarray(view)
            # Get an array from the view that is a *copy* of the original
            # vector.
            value_copy = np.array(view)

    Warning:

        As illustrated above, if you use ``np.array(view)``, then it will
        provide a *copied* array from the view. If you want an *aliased* array
        from the view, then use operations like ``view[:]``,
        ``np.asarray(view)``, or ``np.array(view, copy=False)``.

    For more details, see ``NamedViewBase``.
    """
    base_cls = (NamedViewBase,)
    if sanitize_field_names:
        fields = [_sanitize_field_name(f) for f in fields]
    assert len(set(fields)) == len(fields), "Field names must be unique"
    type_dict = dict(_fields=tuple(fields))
    for i, field in enumerate(fields):
        type_dict[field] = NamedViewBase._item_property(i)
    cls = type(name, base_cls, type_dict)
    return cls



import flags
import numpy
import pydrake.common
import pydrake.systems.framework
import pydrake.systems.sensors
from typing import ClassVar, overload

class BaseField:
    __members__: ClassVar[dict] = ...  # read-only
    __entries: ClassVar[dict] = ...
    kNone: ClassVar[BaseField] = ...
    kNormals: ClassVar[BaseField] = ...
    kRGBs: ClassVar[BaseField] = ...
    kXYZs: ClassVar[BaseField] = ...
    def __init__(self, value: int) -> None: ...
    def __and__(self, other: object) -> object: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __gt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __invert__(self) -> object: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __or__(self, other: object) -> object: ...
    def __rand__(self, other: object) -> object: ...
    def __ror__(self, other: object) -> object: ...
    def __rxor__(self, other: object) -> object: ...
    def __xor__(self, other: object) -> object: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class DepthImageToPointCloud(pydrake.systems.framework.LeafSystem):
    def __init__(self, camera_info: pydrake.systems.sensors.CameraInfo, pixel_type: pydrake.systems.sensors.PixelType = ..., scale: float = ..., fields: int = ...) -> None: ...
    def camera_pose_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def color_image_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def depth_image_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def point_cloud_output_port(self) -> pydrake.systems.framework.OutputPort: ...

class Fields:
    def __init__(self, base_fields: int) -> None: ...
    def base_fields(self) -> int: ...
    def has_base_fields(self) -> bool: ...
    def __and__(self, arg0: Fields) -> Fields: ...
    def __eq__(self, arg0: Fields) -> bool: ...
    def __ne__(self, arg0: Fields) -> bool: ...
    def __or__(self, arg0: Fields) -> Fields: ...

class PointCloud:
    C: ClassVar[type[numpy.uint8]] = ...
    D: ClassVar[type[numpy.float32]] = ...
    T: ClassVar[type[numpy.float32]] = ...
    kDefaultValue: ClassVar[float] = ...
    @overload
    def __init__(self, new_size: int = ..., fields: Fields = ...) -> None: ...
    @overload
    def __init__(self, other: PointCloud) -> None: ...
    def Crop(self, lower_xyz: numpy.ndarray[numpy.float32[3, 1]], upper_xyz: numpy.ndarray[numpy.float32[3, 1]]) -> PointCloud: ...
    def EstimateNormals(self, radius: float, num_closest: int, parallelize: pydrake.common.Parallelism = ...) -> bool: ...
    def FlipNormalsTowardPoint(self, p_CP: numpy.ndarray[numpy.float32[3, 1]]) -> None: ...
    @staticmethod
    def IsDefaultValue(value: float) -> bool: ...
    @staticmethod
    def IsInvalidValue(value: float) -> bool: ...
    def SetFields(self, new_fields: Fields, skip_initialize: bool = ...) -> None: ...
    def SetFrom(self, other: PointCloud) -> None: ...
    def VoxelizedDownSample(self, voxel_size: float, parallelize: pydrake.common.Parallelism = ...) -> PointCloud: ...
    def fields(self) -> Fields: ...
    def has_normals(self) -> bool: ...
    def has_rgbs(self) -> bool: ...
    def has_xyzs(self) -> bool: ...
    def mutable_normal(self, i: int) -> numpy.ndarray[numpy.float32[3, 1], flags.writeable]: ...
    def mutable_normals(self) -> numpy.ndarray[numpy.float32[3, n], flags.writeable, flags.f_contiguous]: ...
    def mutable_rgb(self, i: int) -> numpy.ndarray[numpy.uint8[3, 1], flags.writeable]: ...
    def mutable_rgbs(self) -> numpy.ndarray[numpy.uint8[3, n], flags.writeable, flags.f_contiguous]: ...
    def mutable_xyz(self, i: int) -> numpy.ndarray[numpy.float32[3, 1], flags.writeable]: ...
    def mutable_xyzs(self) -> numpy.ndarray[numpy.float32[3, n], flags.writeable, flags.f_contiguous]: ...
    def normal(self, i: int) -> numpy.ndarray[numpy.float32[3, 1]]: ...
    def normals(self) -> numpy.ndarray[numpy.float32[3, n], flags.f_contiguous]: ...
    def resize(self, new_size: int) -> None: ...
    def rgb(self, i: int) -> numpy.ndarray[numpy.uint8[3, 1]]: ...
    def rgbs(self) -> numpy.ndarray[numpy.uint8[3, n], flags.f_contiguous]: ...
    def size(self) -> int: ...
    def xyz(self, i: int) -> numpy.ndarray[numpy.float32[3, 1]]: ...
    def xyzs(self) -> numpy.ndarray[numpy.float32[3, n], flags.f_contiguous]: ...
    def __copy__(self) -> PointCloud: ...
    def __deepcopy__(self, arg0: dict) -> PointCloud: ...

class PointCloudToLcm(pydrake.systems.framework.LeafSystem):
    def __init__(self, frame_name: str = ...) -> None: ...

def Concatenate(clouds: list[PointCloud]) -> PointCloud: ...



"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

import drake

class lcmt_robot_plan(object):

    __slots__ = ["utime", "num_states", "plan"]

    __typenames__ = ["int64_t", "int32_t", "drake.lcmt_robot_state"]

    __dimensions__ = [None, None, ["num_states"]]

    def __init__(self):
        self.utime = 0
        """ LCM Type: int64_t """
        self.num_states = 0
        """ LCM Type: int32_t """
        self.plan = []
        """ LCM Type: drake.lcmt_robot_state[num_states] """

    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_robot_plan._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        buf.write(struct.pack(">qi", self.utime, self.num_states))
        for i0 in range(self.num_states):
            assert self.plan[i0]._get_packed_fingerprint() == drake.lcmt_robot_state._get_packed_fingerprint()
            self.plan[i0]._encode_one(buf)

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_robot_plan._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_robot_plan._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_robot_plan()
        self.utime, self.num_states = struct.unpack(">qi", buf.read(12))
        self.plan = []
        for i0 in range(self.num_states):
            self.plan.append(drake.lcmt_robot_state._decode_one(buf))
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_robot_plan in parents: return 0
        newparents = parents + [lcmt_robot_plan]
        tmphash = (0x819ba3bc0a8b9ad2+ drake.lcmt_robot_state._get_hash_recursive(newparents)) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_robot_plan._packed_fingerprint is None:
            lcmt_robot_plan._packed_fingerprint = struct.pack(">Q", lcmt_robot_plan._get_hash_recursive([]))
        return lcmt_robot_plan._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_robot_plan._get_packed_fingerprint())[0]




"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

class lcmt_robot_state(object):

    __slots__ = ["utime", "num_joints", "joint_name", "joint_position"]

    __typenames__ = ["int64_t", "int16_t", "string", "float"]

    __dimensions__ = [None, None, ["num_joints"], ["num_joints"]]

    def __init__(self):
        self.utime = 0
        """ LCM Type: int64_t """
        self.num_joints = 0
        """ LCM Type: int16_t """
        self.joint_name = []
        """ LCM Type: string[num_joints] """
        self.joint_position = []
        """ LCM Type: float[num_joints] """

    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_robot_state._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        buf.write(struct.pack(">qh", self.utime, self.num_joints))
        for i0 in range(self.num_joints):
            __joint_name_encoded = self.joint_name[i0].encode('utf-8')
            buf.write(struct.pack('>I', len(__joint_name_encoded)+1))
            buf.write(__joint_name_encoded)
            buf.write(b"\0")
        buf.write(struct.pack('>%df' % self.num_joints, *self.joint_position[:self.num_joints]))

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_robot_state._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_robot_state._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_robot_state()
        self.utime, self.num_joints = struct.unpack(">qh", buf.read(10))
        self.joint_name = []
        for i0 in range(self.num_joints):
            __joint_name_len = struct.unpack('>I', buf.read(4))[0]
            self.joint_name.append(buf.read(__joint_name_len)[:-1].decode('utf-8', 'replace'))
        self.joint_position = struct.unpack('>%df' % self.num_joints, buf.read(self.num_joints * 4))
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_robot_state in parents: return 0
        tmphash = (0xc267d8b916ac5eac) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_robot_state._packed_fingerprint is None:
            lcmt_robot_state._packed_fingerprint = struct.pack(">Q", lcmt_robot_state._get_hash_recursive([]))
        return lcmt_robot_state._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_robot_state._get_packed_fingerprint())[0]

